==> dasu_3.02.jar: edu/northwestern/dasu/stats/DownloadStats.java <==
/**
 * Dasu Project
 *
 * File:         DownloadStats.java
 * RCS:          $Id$
 * Description:  DownloadStats class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 10, 2007 at 1:39:15 PM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;

import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadAnnounceResult;
import org.gudy.azureus2.plugins.download.DownloadListener;
import org.gudy.azureus2.plugins.download.DownloadManager;
import org.gudy.azureus2.plugins.download.DownloadScrapeResult;
import org.gudy.azureus2.plugins.download.DownloadTrackerListener;
import org.gudy.azureus2.plugins.peers.PeerManager;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.timer.ITimerEvent;
import edu.northwestern.dasu.timer.ITimerEventPerformer;
import edu.northwestern.dasu.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The DownloadStats class manages per-download stats
 */
public class DownloadStats  implements ITimerEventPerformer, DownloadTrackerListener {

	public static final int MAX_DSD_ENTRIES = 100;
	public static long UPDATE_INTERVAL = 30 * 1000;
	public static long STATIC_UPDATE_INTERVAL = 30*60*1000;
	long lastStaticUpdateTime = -1;
	private static DownloadStats self;
	private int selfId = -1;
	protected HashMap<Download, DownloadScrapeResult> scrapeResults = 
		new HashMap<Download, DownloadScrapeResult>();
	
	public HashMap<String, DownloadStatsSummary> dlStats = 
		new HashMap<String, DownloadStatsSummary>();
	
	
	public synchronized static DownloadStats getInstance(){
		if (self==null){
			self = new DownloadStats();
		}
		return self;
	}

	public static class DownloadStatsSummary{

		public Download download;
		public long downloadingTime;
		public long seedingTime;
		public int maxDownloadRateLimit;
		public int maxUploadLimit;
		public ArrayList<DownloadStatsDynamic> dynamicStats;
		public long totalUploaded;
		public long totalDownloaded;
		public long size;
		public int downloadId;
		public int priority;
		public long pieceSize;
		public DownloadListener dlist;
		public String downloadName;

		public DownloadStatsSummary(Download d) {
			this.download = d;
			if (d.getTorrent()!=null)
				this.pieceSize = d.getTorrent().getPieceSize();
			else this.pieceSize = -1;
			downloadId =getId(d);
			dynamicStats = new ArrayList<DownloadStatsDynamic>();
			dlist= new DownloadListener(){

				public void positionChanged(Download download, int oldPosition, int newPosition) {
					
					
				}

				public void stateChanged(Download download, int old_state, int new_state) {
					if ((new_state == Download.ST_DOWNLOADING) ||
			                (new_state == Download.ST_SEEDING)) {
			            // taken care of by periodic handler
			        } else if ((new_state == Download.ST_QUEUED) ||
			                (new_state == Download.ST_STOPPED)) {
			            flush();
			            stop();
			        }
					
				}};
			d.addListener(dlist);
			
			downloadName = download.getName();

			if (d.getTorrent()!=null)
				size = roundSize(download.getTorrent().getSize());
			
		}

		private int getId(Download d) {
			MessageDigest md;
			try {
				md = MessageDigest.getInstance ("SHA");
			
			
		        md.update(d.getName().getBytes());
		        md.update(Util.convertLong(d.getCreationTime()));
		        byte hash[] = md.digest();
		        String s = new String(hash);
		        return s.hashCode();
			} catch (NoSuchAlgorithmException e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
			}
			return (int)(d.getName().hashCode() + d.getCreationTime());
		}
		
		protected void summarize(){
			
			//System.out.println("DEBUG: DownloadStats: summary: " + downloadId);
			
			if (DasuConfiguration.getInstance().isRecordStats()){
				Main.getReporter().insertDownloadStatSummary(
					DownloadStats.getInstance().getSelfId(), 
					downloadId,
					Util.currentGMTTime(), 
					downloadingTime, seedingTime, maxDownloadRateLimit, maxUploadLimit, 
					totalUploaded, totalDownloaded, size, pieceSize);
			}
		}

		protected void stop() {
			update();
			flush();

			
			DownloadStats.getInstance().removeDownload(download);
			download.removeListener(dlist);
			dynamicStats = null;
			download = null;
			
		}

		protected void flush() {
			synchronized (dynamicStats){
				LinkedList<DownloadStatsDynamic> toRemove = new 
				LinkedList<DownloadStatsDynamic>();
                // TODO synchronize
				for (DownloadStatsDynamic dsd: dynamicStats){
					if (dsd.flush()){
						toRemove.add(dsd);
					}
				}
				// just clearing this now because duplicate data is 
				// not helping
				dynamicStats.clear();
			}
			Main.getReporter().disconnect();
			
		}

		public void update() {
			
//			System.out.println("DEBUG: DownloadStats: \n\n\nupdate() " + 
//					+ this.downloadId + " " + download.getMaximumDownloadKBPerSecond() 
//					+ " " + download.getUploadRateLimitBytesPerSecond() + "\n\n\n\n");
			
			if (download.getState()== Download.ST_DOWNLOADING ||
			     download.getState() == Download.ST_SEEDING){
				downloadingTime = download.getStats().getSecondsDownloading();
				seedingTime = download.getStats().getSecondsOnlySeeding();
				maxDownloadRateLimit = download.getMaximumDownloadKBPerSecond();
				maxUploadLimit = download.getUploadRateLimitBytesPerSecond();
				totalUploaded = download.getStats().getUploaded();
				if (download.getStats().getCompleted()>900)
					totalDownloaded = roundSize(download.getStats().getDownloaded());
				else totalDownloaded = download.getStats().getDownloaded();
				priority = download.getIndex();
				
				DownloadStatsDynamic dsd = new DownloadStatsDynamic();
				dsd.update(this);
				addEntry(dsd);
			}
			
		}


		/**
		 * Adds an entry to the cache and ensures that the 
		 * cache is not taking up too much memory.
		 * @param dsd the entry to add
		 */
		private void addEntry(DownloadStatsDynamic dsd) {
			
			//System.out.print("downloadStatDynamic5!! FireUp Interference update? " + dsd.dss.downloadId);
			//System.out.println(" completed: " + dsd.completed + " downloaded: " + dsd.downloaded + " dRate: " + dsd.downloadRate);
			
			if (dynamicStats==null) return;
			synchronized(dynamicStats){
			if (dynamicStats.size()>=MAX_DSD_ENTRIES){
				flush();
			}
			
			if (dynamicStats.size()>=MAX_DSD_ENTRIES)			
				dynamicStats.remove(0);
			} 
			dynamicStats.add(dsd);
			
		}

	}
	
	public static class DownloadStatsDynamic{

		public DownloadStatsSummary dss;
		public long timeRecorded;
		public int numPeers;
		public float availability;
		public int completed;
		public int discarded;
		public int downloadRate;
		public int shareRatio;
		public int uploadRate;
		public int numKnownNonSeeds;
		public int numKnownSeeds;
		public int totalNumKnownNonSeeds=-1;
		public int totalNumKnownSeeds=-1;
		public int state;
		public int priority;
		public long downloaded;
		public long uploaded;
		public int position;
		public long hashFails=0;
		public int connectedSeeds = -1;
		public int connectedLeechers = -1;
		public int numPendingPeers = -1;
		public long swarmPeerSpeed = -1;

		public void update(DownloadStatsSummary dss) {
			
			//System.out.println("downloadStatsSummary!! FireUp Interference update? " + dss.downloadId);
			
			this.dss = dss;
			Download download = dss.download;
			timeRecorded = Util.currentGMTTime();
			availability = download.getStats().getAvailability();
			completed = download.getStats().getCompleted();
			discarded = (int) download.getStats().getDiscarded();
			hashFails = download.getStats().getHashFails();
			downloadRate = (int) download.getStats().getDownloadAverage();
			shareRatio = download.getStats().getShareRatio();
			uploadRate = (int) download.getStats().getUploadAverage();
			numKnownNonSeeds = download.getLastAnnounceResult().getNonSeedCount();
			numKnownSeeds = download.getLastAnnounceResult().getSeedCount();
			totalNumKnownNonSeeds = download.getLastScrapeResult().getNonSeedCount();
			totalNumKnownSeeds = download.getLastScrapeResult().getSeedCount();
			state = download.getState();
			priority = download.getIndex();
			position = download.getPosition();
			if (download.getStats().getCompleted()>900)
				downloaded = roundSize(download.getStats().getDownloaded());
			else downloaded = download.getStats().getDownloaded();
			// only hide precise upload value if it is close to the 
			// size of the file (prevents reverse engineering to find 
			// file size, probably can be even less strict but I'm erring 
			// on the side of privacy for users)
			if (Math.abs(shareRatio-1000) <100)
				uploaded = roundSize(download.getStats().getUploaded());
			else uploaded = download.getStats().getUploaded();
			PeerManager pm = download.getPeerManager();			
			if (pm!=null){
				numPeers = pm.getPeers().length;
				connectedSeeds = pm.getStats().getConnectedSeeds();
				connectedLeechers = pm.getStats().getConnectedLeechers();
				numPendingPeers = pm.getPendingPeers().length;
			}
			
			for (Object obj : Main.getGlobalManager().getDownloadManagers()) {
				org.gudy.azureus2.core3.download.DownloadManager dm = (org.gudy.azureus2.core3.download.DownloadManager) obj;
				if (dm.getDisplayName().equals(download.getName()))
					swarmPeerSpeed = dm.getStats().getTotalAveragePerPeer();
			}

		}
		
		public boolean flush(){
			if (!DasuConfiguration.getInstance().isRecordStats()) return true;
			
			return Main.getReporter().insertDownloadStatDynamic(
					DownloadStats.getInstance().getSelfId(), 
					dss.downloadId, timeRecorded, numPeers, availability,
					completed, discarded, downloadRate, uploadRate, shareRatio, 
					numKnownNonSeeds, numKnownSeeds, state, priority,
					downloaded, uploaded, position, totalNumKnownNonSeeds, 
					totalNumKnownSeeds, hashFails, connectedSeeds, connectedLeechers, swarmPeerSpeed);
		}
	}
	
	/**
	 * Perform periodic data-collections
	 */
	public void perform(ITimerEvent event) {
		
		if (!DasuConfiguration.getInstance().isRecordStats() )
			return;
		
		DownloadManager dm = Main.getPluginInterface().getDownloadManager();
		
		Download dls[] = dm.getDownloads();
		for (Download d : dls){		
			getDownloadStats(d).update();
			
		}
		
		if (Util.currentGMTTime()-lastStaticUpdateTime >= STATIC_UPDATE_INTERVAL){
			
			for (Download d : dls){	
				if (d!=null && getDownloadStats(d)!=null){
					getDownloadStats(d).update();
					getDownloadStats(d).summarize();
				}

			}

			lastStaticUpdateTime = Util.currentGMTTime();
		}
		
		

	}
	
	/**
	 * Remove the download from the cache of download objects
	 * @param download
	 */
	public void removeDownload(Download download) {
		synchronized (dlStats){
			dlStats.remove(download.getName());
		}
		
	}

	/**
	 * Get the stats object associated with the specified Download object.
	 * @param d the download object
	 * @return
	 */
	private DownloadStatsSummary getDownloadStats(Download d) {
		DownloadStatsSummary stat = null;
		if (d.getName()!=null) stat = dlStats.get(d.getName());		
		if (stat == null){
			stat = new DownloadStatsSummary(d);
			synchronized (dlStats){
				if (d.getName()!=null)
					dlStats.put(d.getName(), stat);
			}
		}
		return stat;
	}
	
	/**
	 * Returns the database identifier for this node.
	 * @return
	 */
	public int getSelfId(){
		if (selfId == -1){
			selfId  = Statistics.getInstance().getSelfId();
		}
		return selfId;
	}

	/**
	 * Returns the periodicity for calling this object's perform method
	 * @return
	 */
	public static long getInterval() {
		return UPDATE_INTERVAL;
	}

	public void stop() {
		DownloadManager dm = Main.getPluginInterface().getDownloadManager();
		
		Download dls[] = dm.getDownloads();
		
		for (Download d : dls){	
			if (d!=null && getDownloadStats(d)!=null){
				getDownloadStats(d).summarize();
			}

		}
			
		//MainGeneric.getReporter().flush();
		
		int i = 0;
		for (Download d : dls){	
			if (d!=null && getDownloadStats(d)!=null){
				getDownloadStats(d).stop();
			}
//			if (i==0) {
//				try {
//				Thread.sleep(28*1000);
//				} catch (InterruptedException e){
//					e.printStackTrace();
//				}
//			}
//			i++;
		}
		dlStats = null;
		self = null;
		
	}

	public void flush() {
		DownloadManager dm = Main.getPluginInterface().getDownloadManager();
		if (dm==null) return;
		Download dls[] = dm.getDownloads();
		for (Download d : dls){		
			getDownloadStats(d).flush();
			
		}
		
		
	}

	public static void setRecordingInterval(int interval) {
		UPDATE_INTERVAL = interval;
		
	}
	

	/**
	 * Rounds the size of the download.
	 * 
	 * @param dlSize
	 * @return
	 */
	public static long roundSize(long dlSize) {
		if (dlSize < 10000) return dlSize;
		else if (dlSize < 100000) return 1000* (dlSize/1000);
		else if (dlSize < 1000000) return 10000* (dlSize/10000); 
		else if (dlSize < 10000000) return 100000* (dlSize/100000); 
		else if (dlSize < 100000000) return 1000000* (dlSize/1000000);
		else return 10000000* (dlSize/10000000); 
	}

	public void announceResult(DownloadAnnounceResult result) {
		
	}

	public void scrapeResult(DownloadScrapeResult result) {
		scrapeResults.put(result.getDownload(), result);
		
	}

}

==> dasu_4.11.jar: edu/northwestern/dasu/stats/DownloadStats.java <==
/**
 * Dasu Project
 *
 * File:         DownloadStats.java
 * RCS:          $Id$
 * Description:  DownloadStats class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 10, 2007 at 1:39:15 PM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadAnnounceResult;
import org.gudy.azureus2.plugins.download.DownloadListener;
import org.gudy.azureus2.plugins.download.DownloadManager;
import org.gudy.azureus2.plugins.download.DownloadScrapeResult;
import org.gudy.azureus2.plugins.download.DownloadTrackerListener;
import org.gudy.azureus2.plugins.peers.PeerManager;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.timer.ITimerEvent;
import edu.northwestern.dasu.timer.ITimerEventPerformer;
import edu.northwestern.dasu.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The DownloadStats class manages per-download stats
 */
public class DownloadStats extends FactTemplate implements ITimerEventPerformer, DownloadTrackerListener {

	public static final int MAX_DSD_ENTRIES = 100;
	public static long UPDATE_INTERVAL = 30 * 1000;
	public static long STATIC_UPDATE_INTERVAL = 30*60*1000;
	long lastStaticUpdateTime = -1;
	private static DownloadStats self;
	protected HashMap<Download, DownloadScrapeResult> scrapeResults = 
		new HashMap<Download, DownloadScrapeResult>();
	
	public HashMap<String, DownloadStatsSummary> dlStats = 
		new HashMap<String, DownloadStatsSummary>();
	
	
	public synchronized static DownloadStats getInstance(){
		if (self==null){
			self = new DownloadStats();
		}
		return self;
	}

	public class DownloadStatsSummary extends FactTemplate{

		private transient Download download;
		private transient DownloadListener dlist;
		public long downloadingTime;
		public long seedingTime;
		public int maxDownloadRateLimit;
		public int maxUploadLimit;
		private ArrayList<DownloadStatsDynamic> dynamicStats;
		public long totalUploaded;
		public long totalDownloaded;
		public long size;
		public int downloadId;
		public int priority;
		public long pieceSize;		
		private String downloadName;
		public long sumarizeTimeStamp = -1l;

		public DownloadStatsSummary(Download d) {
			
			download = d;
			pieceSize = d.getTorrent() != null ? d.getTorrent().getPieceSize() : -1; 			
			downloadId = Main.getId(d);
			dynamicStats = new ArrayList<DownloadStatsDynamic>();
			downloadName = download.getName();

			if (d.getTorrent()!=null)
				size = roundSize(download.getTorrent().getSize());

			dlist= new DownloadListener(){

				public void positionChanged(Download download, int oldPosition, int newPosition) {
					
				}
				
				public void stateChanged(Download download, int old_state, int new_state) {
					
					if ((new_state == Download.ST_DOWNLOADING) ||
			                (new_state == Download.ST_SEEDING)) {
			            // taken care of by periodic handler
			        } else if ((new_state == Download.ST_QUEUED) ||
			                (new_state == Download.ST_STOPPED)) {
			        	
			        	updateStopped();						
			        }
				}
				
			};
				
			d.addListener(dlist);
			
		}

		protected void summarize(){
			
			sumarizeTimeStamp = Util.currentGMTTime(); 
			Statistics.getInstance().addReportableFact(this);
			
		}

		protected void stop() {
			
			
			if ( DownloadStats.getInstance().dlStats.containsKey(download) ) {
				update();			
				flush();				
				DownloadStats.getInstance().removeDownload(download);
				download.removeListener(dlist);
			}
			else {
				//already removed by the listener
				flush();
			}
			
			//dynamicStats = null;
			//download = null;
			
		}
		
		public void updateStopped() {
			
			DownloadStats.getInstance().removeDownload(download);
			download.removeListener(dlist);

		}

		protected void flush() {
			
			synchronized (dynamicStats) {
				
				Iterator<DownloadStatsDynamic> iter = dynamicStats.iterator();
				while ( iter.hasNext() ) {
					DownloadStatsDynamic dsd = iter.next();
					dsd.flush();
					iter.remove();
				}
			}
		}
		
		public void update() {
			
			if (download.getState()== Download.ST_DOWNLOADING || 
					download.getState() == Download.ST_SEEDING) {
				
				downloadingTime = download.getStats().getSecondsDownloading();
				seedingTime = download.getStats().getSecondsOnlySeeding();
				maxDownloadRateLimit = download.getMaximumDownloadKBPerSecond();
				maxUploadLimit = download.getUploadRateLimitBytesPerSecond();
				totalUploaded = download.getStats().getUploaded();
				
				if (download.getStats().getCompleted()>900)
					totalDownloaded = roundSize(download.getStats().getDownloaded());
				else 
					totalDownloaded = download.getStats().getDownloaded();
				
				priority = download.getIndex();
				
				DownloadStatsDynamic dsd = new DownloadStatsDynamic();
				dsd.update(this);
				addEntry(dsd);
				
			}
			
			else if ( download.getState() == Download.ST_QUEUED ||
	                download.getState() == Download.ST_STOPPED )  {
								
				stop();
			}
			
		}

		/**
		 * Adds an entry to the cache and ensures that the 
		 * cache is not taking up too much memory.
		 * @param dsd the entry to add
		 */
		private void addEntry(DownloadStatsDynamic dsd) {
			
			//System.out.print("downloadStatDynamic5!! FireUp Interference update? " + dsd.dss.downloadId);
			//System.out.println(" completed: " + dsd.completed + " downloaded: " + dsd.downloaded + " dRate: " + dsd.downloadRate);
			
			if (dynamicStats==null) 
				return;
			
			synchronized(dynamicStats){
				
				if (dynamicStats.size()>=MAX_DSD_ENTRIES)
					flush();
				
				
				if (dynamicStats.size()>=MAX_DSD_ENTRIES)			
					dynamicStats.remove(0);
			} 
			
			dynamicStats.add(dsd);
			
		}

	}
	
	public class DownloadStatsDynamic extends FactTemplate{

		//private DownloadStatsSummary dss;
		public int downloadId;
		public long timeRecorded;
		public int numPeers;
		public float availability;
		public int completed;
		public int discarded;
		public int downloadRate;
		public int shareRatio;
		public int uploadRate;
		public int numKnownNonSeeds;
		public int numKnownSeeds;
		public int totalNumKnownNonSeeds=-1;
		public int totalNumKnownSeeds=-1;
		public int state;
		public int priority;
		public long downloaded;
		public long uploaded;
		public int position;
		public long hashFails=0;
		public int connectedSeeds = -1;
		public int connectedLeechers = -1;
		public int numPendingPeers = -1;
		public long swarmPeerSpeed = -1;

		public void update(DownloadStatsSummary dss) {
			
			//this.dss = dss;
			downloadId = dss.downloadId;
			Download download = dss.download;
			timeRecorded = Util.currentGMTTime();
			availability = download.getStats().getAvailability();
			completed = download.getStats().getCompleted();
			discarded = (int) download.getStats().getDiscarded();
			hashFails = download.getStats().getHashFails();
			downloadRate = (int) download.getStats().getDownloadAverage();
			shareRatio = download.getStats().getShareRatio();
			uploadRate = (int) download.getStats().getUploadAverage();
			numKnownNonSeeds = download.getLastAnnounceResult().getNonSeedCount();
			numKnownSeeds = download.getLastAnnounceResult().getSeedCount();
			totalNumKnownNonSeeds = download.getLastScrapeResult().getNonSeedCount();
			totalNumKnownSeeds = download.getLastScrapeResult().getSeedCount();
			state = download.getState();
			priority = download.getIndex();
			position = download.getPosition();
			
			if (download.getStats().getCompleted()>900)
				downloaded = roundSize(download.getStats().getDownloaded());
			else 
				downloaded = download.getStats().getDownloaded();
			// only hide precise upload value if it is close to the 
			// size of the file (prevents reverse engineering to find 
			// file size, probably can be even less strict but I'm erring 
			// on the side of privacy for users)
			if (Math.abs(shareRatio-1000) <100)
				uploaded = roundSize(download.getStats().getUploaded());
			else 
				uploaded = download.getStats().getUploaded();
			
			PeerManager pm = download.getPeerManager();			
			if (pm!=null){
				numPeers = pm.getPeers().length;
				connectedSeeds = pm.getStats().getConnectedSeeds();
				connectedLeechers = pm.getStats().getConnectedLeechers();
				numPendingPeers = pm.getPendingPeers().length;
			}
			
			for (Object obj : Main.getGlobalManager().getDownloadManagers()) {
				org.gudy.azureus2.core3.download.DownloadManager dm = (org.gudy.azureus2.core3.download.DownloadManager) obj;
				if (dm.getDisplayName().equals(download.getName()))
					swarmPeerSpeed = dm.getStats().getTotalAveragePerPeer();
			}

		}
		
		public void flush(){
			Statistics.getInstance().addReportableFact(this);			
		}
	}
	
	/**
	 * Perform periodic data-collections
	 */
	public void perform(ITimerEvent event) {
		
		DownloadManager dm = Main.getPluginInterface().getDownloadManager();
		
		Download dls[] = dm.getDownloads();
		for (Download d : dls){		
			getDownloadStats(d).update();
			
		}
		
		if (Util.currentGMTTime()-lastStaticUpdateTime >= STATIC_UPDATE_INTERVAL){
			
			for (Download d : dls){	
				if (d!=null && getDownloadStats(d)!=null){
					getDownloadStats(d).update();
					getDownloadStats(d).summarize();
				}

			}

			lastStaticUpdateTime = Util.currentGMTTime();
		}
		
	}
	
	/**
	 * Remove the download from the cache of download objects
	 * @param download
	 */
	public void removeDownload(Download download) {
		synchronized (dlStats){
			dlStats.remove(download.getName());
		}
		
	}

	/**
	 * Get the stats object associated with the specified Download object.
	 * @param d the download object
	 * @return
	 */
	private DownloadStatsSummary getDownloadStats(Download d) {
		DownloadStatsSummary stat = null;
		if (d.getName()!=null) stat = dlStats.get(d.getName());		
		if (stat == null){
			stat = new DownloadStatsSummary(d);
			synchronized (dlStats){
				if (d.getName()!=null)
					dlStats.put(d.getName(), stat);
			}
		}
		return stat;
	}
	
	/**
	 * Returns the periodicity for calling this object's perform method
	 * @return
	 */
	public static long getInterval() {
		return UPDATE_INTERVAL;
	}

	public void stop() {
		
		flush();
		DownloadManager dm = Main.getPluginInterface().getDownloadManager();
		
		Download dls[] = dm.getDownloads();
		
		for (Download d : dls){	
			if (d!=null && getDownloadStats(d)!=null){
				getDownloadStats(d).summarize();
			}

		}
			
		for (Download d : dls){	
			if (d!=null && getDownloadStats(d)!=null){
				getDownloadStats(d).stop();
			}
		}
		
	}

	public void flush() {
		
		DownloadManager dm = Main.getPluginInterface().getDownloadManager();
		if (dm==null) return;
		Download dls[] = dm.getDownloads();
		for (Download d : dls){		
			getDownloadStats(d).flush();
			
		}
		
		
	}

	public static void setRecordingInterval(int interval) {
		UPDATE_INTERVAL = interval;
		
	}
	

	/**
	 * Rounds the size of the download.
	 * 
	 * @param dlSize
	 * @return
	 */
	public static long roundSize(long dlSize) {
		if (dlSize < 10000) return dlSize;
		else if (dlSize < 100000) return 1000* (dlSize/1000);
		else if (dlSize < 1000000) return 10000* (dlSize/10000); 
		else if (dlSize < 10000000) return 100000* (dlSize/100000); 
		else if (dlSize < 100000000) return 1000000* (dlSize/1000000);
		else return 10000000* (dlSize/10000000); 
	}

	public void announceResult(DownloadAnnounceResult result) {
		
	}

	public void scrapeResult(DownloadScrapeResult result) {
		scrapeResults.put(result.getDownload(), result);
		
	}

}
